/* vi: expandtab:softtabstop=4:tabstop=4:cindent:cino=>4,g4,h4,+4,c1,C4,(4,u4,{0,^-0
 */

/*

Copyright (c) 2008, SFB/TR-55
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __SRC_CUZCO_CONNECTION
#define __SRC_CUZCO_CONNECTION

#include <iostream>
#include <vector>

extern "C" {
#include <superroot_lib.h>
#include <sr_cuzco.h>
#include <stdint.h>
}

using namespace std;

// the status data that we read from the ds2000
struct ds2000_status_t
{
  bool vout_fault, iout_fault, inp_fault, power_not_good, fan_fault,
    unknown_fault, other_fault, mfr_spec_fault, off, busy, vout_ov_fault,
    iout_oc_fault, vin_uv_fault, temp_fault, cml_event, none;
  double vout, vout_cmd, iout, vin, iin, pout, pin, temp1, temp2,
    fan_speed1, fan_speed2;                       ;
};

// represents a connection from frontend to psu/cuzco via superrott
class srcCuzcoConnection
{
  public:
    // constructor/destructor
    srcCuzcoConnection();
    ~srcCuzcoConnection();

    // init/close connection
    int connect(string srcIP);
    int disconnect();
    // set/get current PSU
    int setPSU(int psu);
    int getPSU(int psu);
    // check if the given PSU is present (and set PSU)
    // WARNING: return 1 on success, 0 on failure!
    int checkPSU(int index);

    // the actual functionality
    int powerOn(int psu = -1);
    int powerOff(int psu = -1);
    int getStatus(ds2000_status_t * r);
    int getStatusShort(ds2000_status_t * r, int * warnings);
    int setFan(double speed);
    int setVoltage(double voltage);

  private:
    bool connected;
    string lockfile;
    struct fmod_sr_connection * srcConnection;
    int currentPSU;

    void die(const string& msg);

    int readLinear(double * d, uint8_t pmbus_cmd, bool sgn=true);
    int readDirect(double * d, uint8_t pmbus_cmd);

    // constants

    // I2C slave address of i2c expander on Cuzco
    static const uint16_t cuzco_addr  = 0x19;
    // I2C slave address of ds2000
    static const uint16_t ds2000_addr = 0x38;
    // PMBus commands
    static const uint8_t PMBus_coefficients;
    static const uint8_t PMBus_vout;
    static const uint8_t PMBus_vout_command;
    static const uint8_t PMBus_vin;
    static const uint8_t PMBus_iin;
    static const uint8_t PMBus_iout;
    static const uint8_t PMBus_pout;
    static const uint8_t PMBus_pin;
    static const uint8_t PMBus_temperature_1;
    static const uint8_t PMBus_temperature_2;
    static const uint8_t PMBus_temperature_3;
    static const uint8_t PMBus_fan_speed_1;
    static const uint8_t PMBus_fan_speed_2;
    static const uint8_t PMBus_fan_speed_3;
    static const uint8_t PMBus_fan_speed_4;
    static const uint8_t PMBus_fan_command_1;
    static const uint8_t PMBus_status_word;
    // i2c bit rate in kHz
    static const int i2c_bitrate   = 100;

};

#endif // __SRC_CUZCO_CONNECTION
